#
# -*- coding: iso-8859-15 -*-
#

# 199??? first version by ???
# 201907 updates by JLS:
#     files_in_dir now looks for files in subfolders ending in "-brewerid". optionally, it can also
#+        recurse infinitely into all subfolders
# 202006 updates by JLS: printNlog function, to write info to the screen and/or a log file
# 202110 updates by JLS: check change time of the files

import ConfigParser
import os
from urllib import unquote, splittype, splithost

import re #J used by files_in_dir
import datetime #J used by fall back log in printNlog
import time # used to calculate the current time in seconds since the epoch

#J ------------------------------------------------------------------------------------------------
class ArraySplitter:
        '''
        Iterator que parte un array en trozos de una longitud determinada.
        '''
        def __init__(self, array, step):
                self.array = array
                self.step = step
                self.pos = 0

        def __iter__(self):
                return self

        def next(self):
                if self.pos >= len(self.array):
                        raise StopIteration

                result = self.array[self.pos : self.pos + self.step]
                self.pos += self.step

                return result


#J ------------------------------------------------------------------------------------------------
def check_filename(filename):
        if not os.path.isfile(filename):
                raise ParamException("'" + filename + "' is not a regular file")

#J ------------------------------------------------------------------------------------------------
def check_dirname(dirname):
        if not os.path.isdir(dirname):
                raise ParamException("'" + dirname + "' is not a directory")

#J ------------------------------------------------------------------------------------------------
def check_input(filename):
        if not os.path.isdir(filename) and not os.path.isfile(filename):
                raise ParamException("'" + filename + "' is neither a directory nor a file")

#J ------------------------------------------------------------------------------------------------
def check_brewerid(brewerid):
        try:
                return int(brewerid)
        except ValueError:
                raise ParamException("'" + brewerid + "' is not a valid brewer id")

#J ------------------------------------------------------------------------------------------------
def is_ascii(filename):
    # check if a filename is written in ascii, see
    # https://stackoverflow.com/questions/196345/how-to-check-if-a-string-in-python-is-in-ascii
    #
    # 202105 JLS
    try:
        filename.encode("ascii")
        ascii=True
    except:
        ascii=False
    return ascii

#J ------------------------------------------------------------------------------------------------
def files_in_dir(dirname, brewerid, recursive, daysnewer):
        # 201907 updated by JLS: search in some folders or in all subfolders
        dirnames_str=None
 
        timenow=time.mktime(time.localtime())

        if not recursive:
            # give brewerid the correct format
            brewerid='%03d' %brewerid

            # dirs to search into
            dirnames=[dirname, os.path.join(dirname,brewerid)]

            # add Volodya's new subdirs -- assume they always end in "-brewerid"
            vdirnames=[]
            # recurse (infinitely) into subdirs -- this might kill some slow machines
            #dirnames=dirnames+[dp for dp, dn, fn in os.walk(dirname) \
            #        if re.search('-'+brewerid,dp)]

            # go only into first level subfolders
            vdirnames=vdirnames+[os.path.join(dirname,dp) for dp in os.listdir(dirname) \
                    if os.path.isdir(os.path.join(dirname,dp)) and re.search('-'+brewerid,dp)] 
            
            # now go into first level subfolders of the already found first level subfolders
            #+(needed at least for DSP-brewerid)
            for mydirname in vdirnames:
                vdirnames=vdirnames+[os.path.join(mydirname,dp) for dp in os.listdir(mydirname) \
                    if os.path.isdir(os.path.join(mydirname,dp))]

            # join both lists of dirnames
            if vdirnames:
                dirnames=dirnames+vdirnames

            # tell something to the user
            #print "    Folders to process: "+", ".join(dirnames)
            dirnames_str=", ".join(dirnames)

            # gen list of fullfilenames
            result=[]
            for myDir in dirnames:
                if os.path.isdir(myDir):
                    if daysnewer==0:
                        # append would create a list of sublists, + creates a single list
                        result=result+[os.path.join(myDir, fname) for fname in os.listdir(myDir)\
                                if (os.path.isfile(os.path.join(myDir, fname))) and (len(fname)<=32)\
                                and (is_ascii(fname))]
                    else:
                        # only submit the files which have changed in the last daysnewer days
                        result=result+[os.path.join(myDir, fname) for fname in os.listdir(myDir)\
                                if (os.path.isfile(os.path.join(myDir, fname))) and (len(fname)<=32)\
                                and (is_ascii(fname)) and \
                                ((timenow - os.path.getmtime(os.path.join(myDir,fname)))<=daysnewer*86400)]
                    #print str(myDir)
                    #print str(result)

        else: # search in all subfolders   
            result = [os.path.join(dp, f) for dp, dn, fn in os.walk(dirname) for f in fn\
                    if (os.path.isfile(os.path.join(dp, f))) and (len(f)<=32) and (is_ascii(f))]

        return result,dirnames_str


#J ------------------------------------------------------------------------------------------------
class IniException(Exception):
        pass


#J ------------------------------------------------------------------------------------------------
class IniFileNotExist(IniException):
        pass


#J ------------------------------------------------------------------------------------------------
class ParamException(Exception):
        pass


#J ------------------------------------------------------------------------------------------------
class ServerDef:
        '''
        Contiene la descripcin de una seccin de servidor en el fichero de configuracin
        '''
        def __init__(self, url, dbfile, proxy = None):
                self.url = url
                self.dbfile = dbfile
                self.proxy = proxy


#J ------------------------------------------------------------------------------------------------
class Options:
        def __init__(self):
                self.brewerid = None
                self.servers = None
                self.input = None
                self.rundir = None

#J ------------------------------------------------------------------------------------------------
class IniParser:
        '''
        Construye el objeto de opciones ledas del fichero de configuracin.

        Modo de uso:
        iniparser = IniParser('client.ini')
        iniparser.ini_path = '/tmp/clientconfiguration.ini'
        iniparser.parseIni()

        options = iniparser.options
        server_defs = iniparser.server_defs
        '''
        def __init__(self, ini_filename):
                self.ini_filename = ini_filename
                self.ini_path = None                # Si se modifica, se intenta leer el fichero de configuracin del nombre de fichero definido por ini_path.
                self.options = Options()
                self.server_defs = {}                # Diccionario de ServerSection. Clave: url del servidor. Valor: ServerSection.
                self.proxy = None
        
                self.globalSection_str = 'global'
                self.servers_str = 'servers'
                self.brewerid_str = 'brewerid'
                self.input_str = 'input'
                self.rundir_str = 'working_dir'
                self.proxy_str = 'proxy'
                self.noproxy_str = 'noproxy'


        def parseIni(self):
                ini_filename = self.ini_filename
                if not os.path.exists(ini_filename):
                        raise IniFileNotExist('File "%s" does not exists.' % ini_filename)

                if not os.path.isfile(ini_filename):
                        raise IniException('"%s" is not a file.' % ini_filename)        

                global_str = self.globalSection_str

                self.ini = ConfigParser.RawConfigParser()
                home_ini = '~/.%s' % ini_filename
                etc_ini = os.path.join( os.path.sep + 'data', 'etc', '%s' % ini_filename )
                if self.ini_path != None:
                        paths = [ self.ini_path ]
                else:
                        paths = [ini_filename, os.path.expanduser(home_ini), etc_ini]
                self.ini.read(paths)

                self.options.servers = self.getGlobalOption(self.servers_str)
                self.options.brewerid = self.getGlobalOption(self.brewerid_str)
                self.options.input = self.getGlobalOption(self.input_str)
                self.options.rundir = self.getGlobalOption(self.rundir_str)
                self.proxy = self.getGlobalOption(self.proxy_str)
                self.server_defs = self.parseServers()


        def parseServers(self):
                server_sections = [ section for section in self.ini.sections() if section != self.globalSection_str ]
                result = {}
                for section in server_sections:
                        try:
                                url = self.ini.get(section, 'url')
                        except ConfigParser.NoOptionError:
                                raise IniException('url option must be specified for ' + section + ' server')
                        try:
                                dbfile = self.ini.get(section, 'db')
                                #
                                # dbfile est en el directorio rundir.
                                #
                                dbfile = os.path.join(self.options.rundir, dbfile)
                        except ConfigParser.NoOptionError:
                                raise IniException('db option must be specified for ' + section + ' server')

                        #
                        # Si noproxy es falso, se copia el valor proxy de la seccin global.
                        #
                        try:
                                noproxy = self.ini.get(section, 'noproxy')
                                if noproxy == 'yes' or noproxy == '1' or noproxy == 'true':
                                        proxy = None
                                else:
                                        proxy = self.proxy
                        except ConfigParser.NoOptionError:
                                proxy = self.proxy

                        server_def = ServerDef(url, dbfile, proxy)
                        result[section] = server_def
                        
                return result


        def getGlobalOption(self, optionName):
                try:
                        return self.ini.get(self.globalSection_str, optionName)
                except ConfigParser.NoOptionError:
                        return None


#J ------------------------------------------------------------------------------------------------
def split_proxy_URL(proxy):
        if proxy is None:
                return None, None, None

        try:
                if proxy[:7] != 'http://':  # Ensures proxy string begins with 'http://'
                        proxy = 'http://' + proxy
        except:
                pass

        proxy_username = proxy_password = None

        urltype, r_type = splittype(proxy)
        proxy, XXX = splithost(r_type)
        if '@' in proxy:
                proxy_username, proxy = proxy.split('@', 1)
                if ':' in proxy_username:
                        proxy_username, proxy_password = proxy_username.split(':', 1)

        return proxy, proxy_username, proxy_password

#J ------------------------------------------------------------------------------------------------
def printNlog(msg,logFile,where='b'):
    # print to the screen and/or write to a log file
    # 202006 JLS

    if where=="s": # only write to the screen
        print msg

    elif not logFile: # log file not defined, write to generic log
        print msg

        now=datetime.datetime.now().strftime('%H:%M:%S %Y-%m-%d')
        fallBackLog=open('refresh_error.txt', 'a')
        fallBackLog.write(now+": "+msg+'\n')
        fallBackLog.close()

    elif where=="l": # only write to the log file
        logFile.write(msg+'\n')

    else: # write both to the screen and the log file
        print msg

        logFile.write(msg+'\n')


